/**
 * Clase   que manipula la conexion del lado del Cliente.
 * @author nsanz
 */
import java.io.*;
import javax.swing.text.JTextComponent;
import java.net.*;
import javax.swing.JOptionPane;

public class ConexionCliente extends AbstractConexion {
    /**
     * Socket de conexion hacia el servidor
     */
    private Socket clientSocket;
    /**
     * Constructor de la conexion desde el lado del cliente.
     * @param componenteTexto Componente correspondiente al cuadro de texto de edicion.
     * @param archivo Nombre de archivo.
     */
    public ConexionCliente(JTextComponent componenteTexto, String archivo) {
        super(componenteTexto, archivo);
    }
/**
     * Inicializa los parametros de la conexion
     * @param ip Direccion IP a la que se desea conectar, null en caso de ser servidor.
     * @param puerto
     */
   
    public void configurarConexion(String ip, int puerto) {
        super.ip = ip;
        super.puerto = puerto;
    }
/**
     * Implementacion de Runnable, para la creacion de nuevas hebras para manejar las conexione. Se encarga de inicializar la coenxion.
     */
    public void run() {
        try {
            clientSocket = new Socket(ip, puerto);

            // Se env�a un mensaje de petici�n de fichero.
            ObjectOutputStream oos = new ObjectOutputStream(clientSocket.getOutputStream());
            MensajePide mensaje = new MensajePide();
            mensaje.opcion = MensajePide.Opciones.PEDIR_INICIAL;
            oos.writeObject(mensaje);

            // Se crea un ObjectInputStream del socket para leer los mensajes
            // que contienen el fichero.
            ObjectInputStream ois = new ObjectInputStream(clientSocket.getInputStream());
            MensajeTomaFichero mensajeRecibido;
            Object mensajeAux;

            do {
                // Se lee el mensaje en una variabla auxiliar
                mensajeAux = ois.readObject();

                // Si es del tipo esperado, se trata
                if (mensajeAux instanceof MensajeTomaFichero && ((MensajeTomaFichero) mensajeAux).opcion == MensajeTomaFichero.Opciones.ENVIAR_INICIAL) {
                    mensajeRecibido = (MensajeTomaFichero) mensajeAux;
                    // Se escribe 
                    //System.out.print(mensajeRecibido.contenidoLinea.toString());
                    componenteTexto.setText(componenteTexto.getText() + mensajeRecibido.contenidoLinea.toString());
                } else// Si no es del tipo esperado, se marca error y se termina
                {// el bucle
                    System.err.println("Mensaje no esperado " + mensajeAux.getClass().getName());
                    break;
                }
            } while (mensajeRecibido.ultimoMensaje);

            MensajeTomaCaracter mensajeRecibidoCaracter;
            BufferedReader br = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
            while (true) {

                mensajeAux = ois.readObject();
                if ((Object) mensajeAux instanceof MensajeTomaCaracter) {
                    mensajeRecibidoCaracter = (MensajeTomaCaracter) mensajeAux;
                    recivirCaracteresEditados(mensajeRecibidoCaracter.c, mensajeRecibidoCaracter.location, mensajeRecibidoCaracter.backSpace);
                } else if (mensajeAux instanceof MensajeFinal && ((MensajeFinal) mensajeAux).terminar == true) {
                    ois.close();
                    clientSocket.close();
                    break;
                }

            }


        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                    componenteTexto, e, "Error en la conexion",
                    JOptionPane.ERROR_MESSAGE);
        }
        componenteTexto.setEditable(false);
        componenteTexto.setEnabled(false);
        
    }
/**
 * Para finalizar la conexion.
 */
    public void cerrarConexion() {
        try {
            clientSocket.close();

        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                    componenteTexto, e, "Error en la conexion",
                    JOptionPane.ERROR_MESSAGE);
        }
    }
/**
 * Accion de recivir los caracteres 
 * @param c Caracter que se va a enviar.
 * @param location Posicion en el texto.
 * @param bs Es true si se borra un caracter. Accion de hacer backspace.
 */

    public void recivirCaracteresEditados(char c, int location, boolean bs) {
        if (bs == true) {
            componenteTexto.select(location, location+1);
            componenteTexto.replaceSelection("");

        } else {
            componenteTexto.select(location, location);
            System.out.println("Location= " + location);
            char[] couple = {c};
            componenteTexto.replaceSelection(new String(couple));
        }
    }
    /**
 * Accion de enviar los caracteres 
 * @param c Caracter que se va a enviar.
 * @param location Posicion en el texto.
 * @param bs Es true si se borra un caracter. Accion de hacer backspace.
 */
    public void enviarCaracteresEditados(char c, int location, boolean bs) {
        System.out.println("Para la version 1.0");
    }

}
